;===============================================================================
;
; Program Name:     TaskTimer()
; Description:      Task Timer
; Requirement(s):   None
; Return Value(s):  None
; Author(s):        Jim Michaels <jmichae3@yahoo.com>
; Create Date:      2007
; Current Date:     3/27/2011
; Version:          1.22
;
; Copyright 2007 Jim Michaels
;
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.
;
;
;
;===============================================================================


#include <GUIConstantsEx.au3>
#include <Constants.au3>
#include <EditConstants.au3>
#include <SQLite.au3>
#include <SQLite.dll.au3>
#Include <GuiComboBox.au3>
$PROGRAM_VERSION="1.11"

Global $dbfilename, $num, $ts, $timer_running, $taskname, $ts, $hDB, $hQuery, $hquery, $aRow
_SQLite_Startup ()
If @error > 0 Then
    MsgBox(16, "SQLite Error", "SQLite.dll Can't be Loaded!")
    Exit - 1
EndIf
$hDB=_SQLite_Open("timertest.sqlitedb"); Open a :memory: database
If @error > 0 Then
    MsgBox(16, "SQLite Error", "Can't Load Database!")
	_SQLite_Shutdown()
    Exit - 1
EndIf
If $SQLITE_OK <> _SQLite_Exec($hDB,"CREATE TABLE IF NOT EXISTS timeentries (entry_id INTEGER CONSTRAINT z PRIMARY KEY AUTOINCREMENT, taskname VARCHAR(100) NOT NULL, elapsed INTEGER, startdate VARCHAR(11), starttime VARCHAR(9));") Then
    MsgBox(0,"SQLite Error","Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
	_SQLite_Close($hDB)
	_SQLite_Shutdown()
	exit -1
endif


$dbfilename="TaskTimerDB.txt"
$taskname=""
$elapsed=0
$startdate=""
$starttime=""

$newtaskname=""
$newelapsed=0
$newstartdate=""
$newstarttime=""

$ts=0
$timer_running=false


func join1($arr, $joinby)
	if (ubound($arr)==2) Then return $arr[1]
	$str=$arr[1]
	for $x=2 to ubound($arr)-1
		$str &= $joinby & $arr[$x]
	next
	return $str
endfunc



func join0($arr, $joinby)
	if (ubound($arr)==1) Then return $arr[0]
	$str=$arr[0]
	for $x=1 to ubound($arr)-1
		$str &= $joinby & $arr[$x]
	next
	return $str
endfunc



func getcombo()
	$a=StringSplit(_GUICtrlComboBox_GetList( $COMBOTask), "|")
	$i=_GUICtrlComboBox_GetCurSel( $COMBOTask )
	if ($CB_ERR==$i) Then
		return ""
	Else
		return StringStripWS($a[$i+1], 1+2)
	endif
endfunc



func setcombo()
	$err=_SQLite_Query ( $hDB, "SELECT DISTINCT taskname FROM timeentries ORDER BY taskname;", $hQuery )
	select
	case $err==$SQLITE_OK
	case $err==-1
		msgbox(0, "SQLite error", "SELECT DISTINCT1 prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
	case $err=1
		msgbox(0, "SQLite error", "SELECT DISTINCT1 problem:Error calling SQLite API 'sqlite3_prepare'")
	case $err=2
		msgbox(0, "SQLite error", "SELECT DISTINCT1 problem: call prevented by safe mode.")
	endselect
	_GUICtrlComboBox_ResetContent( $COMBOTask )
	$f=false
	While _SQLite_FetchData ($hQuery, $aRow) = $SQLITE_OK
		$err=_GUICtrlComboBox_AddString ( $COMBOTask, $aRow[0] )
		if $err==$CB_ERR then msgbox(0,"combo","CB_ERR")
		if $err==$CB_ERRSPACE then msgbox(0,"combo","CB_ERRSPACE")
		;if $err>=0 then msgbox(0,"setcombo","add:"&$err)
		if ($taskname <> "" and $taskname==$aRow[0]) then
			$f=true
		endif
	WEnd
	_SQLite_QueryFinalize ($hQuery)
	;add $taskname to list if possible, valid, and unique
	if ($taskname<>"" and $f==false) then
		$err=_GUICtrlComboBox_AddString ( $COMBOTask, $taskname )
		if $err==$CB_ERR then msgbox(0,"combo","CB_ERR")
		if $err==$CB_ERRSPACE then msgbox(0,"combo","CB_ERRSPACE")
		;if $err>=0 then msgbox(0,"setcombo","add:"&$err)
	endif
endfunc



func setcombodefault($str)
	$i=_GUICtrlComboBox_FindString ( $COMBOTask, $str, 1)
	if ($i <> $CB_ERR) then
		_GUICtrlComboBox_SetCurSel ( $COMBOTask,  $i)
	Else
		msgbox(0,"setcombodefault","oops. index="&$i&": str="&$str)
	endif
endfunc



func eltime($timein)
	return StringFormat("%d:%02d:%02d", int($timein/3600), Mod(int($timein/60), 60), Mod($timein, 60))
endfunc



func toteltime($tasknm)
	$err=_SQLite_Query ( $hDB, "SELECT elapsed FROM timeentries WHERE taskname="&_SQLite_Escape($tasknm)&";", $hquery)
	select
	case $err==$SQLITE_OK
	case $err==-1
		msgbox(0, "SQLite error", "SELECT elapsed prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
	case $err=1
		msgbox(0, "SQLite error", "SELECT elapsed problem:Error calling SQLite API 'sqlite3_prepare'")
	case $err=2
		msgbox(0, "SQLite error", "SELECT elapsed problem: call prevented by safe mode.")
	endselect
	$tot=0
	While _SQLite_FetchData ($hquery, $aRow) = $SQLITE_OK
		$tot += $aRow[0]
	WEnd
	_SQLite_QueryFinalize ($hQuery)
	return eltime($tot)
endfunc


func gentskname()
	$err=_SQLite_Query ( $hDB, "SELECT DISTINCT taskname FROM timeentries ORDER BY taskname;", $hQuery )
	select
	case $err==$SQLITE_OK
	case $err==-1
		msgbox(0, "SQLite error", "SELECT DISTINCT2 prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
	case $err=1
		msgbox(0, "SQLite error", "SELECT DISTINCT2 problem:Error calling SQLite API 'sqlite3_prepare'")
	case $err=2
		msgbox(0, "SQLite error", "SELECT DISTINCT2 problem: call prevented by safe mode.")
	endselect
	_SQLite_QueryFinalize ($hQuery)
	$f=true
	$result=""
	While _SQLite_FetchData ($hQuery, $aRow) = $SQLITE_OK
		if ($f) then ;first time only
			$result= $aRow[0]
			$f=false
		else
			$result &= '|' & $aRow[0]
		endif
	WEnd
	return $result
endfunc



Func readdb($filename)
	if ($filename="") Then
		Return
	EndIf
	$file = FileOpen($filename, 0) ;read
	; Check if file opened for reading OK
	If $file = -1 Then
		;MsgBox(0, "Database File Error", "Unable to open file for reading.")
		return
	EndIf
	$index=0
	$linecount=1
	While 1
		$charptr=1
		$beginptr=0
		$line = FileReadLine($file)
		If @error = -1 Then ExitLoop
		;find first quote
		while ($charptr < stringlen($line) AND stringmid($line, $charptr, 1)<>'"') ;start by finding a quote
			$charptr+=1
		wend

		;if we hit a blank or bogus line, skip the line.
		if ($charptr >= stringlen($line)-1) then continueloop

		;read taskname
		if (stringmid($line, $charptr, 1)=='"') then ;process task name in between the quotes
			$charptr+=1 ;skip past quote
			$beginptr=$charptr
			while (stringmid($line, $charptr, 1) <> '"' AND $charptr <= stringlen($line))
				$charptr+=1
			wend
			;charptr now points to 2nd quote
			$taskname=stringmid($line, $beginptr, $charptr-$beginptr)
		Else
			MsgBox(0, "Database Text File Error:", "Invalid File Format found in task name on line " & $linecount)
		endif

		;skip space(s)
		do
		    $charptr+=1
		until ($charptr > stringlen($line) OR stringmid($line, $charptr, 1)<>' ')

		;read elapsed time
		$beginptr=$charptr
		Do ;skip until space(s)
			$charptr+=1
		until ($charptr > stringlen($line) OR stringmid($line, $charptr, 1)==' ')
		$et=StringMid($line, $beginptr, $charptr-$beginptr)
;		msgbox(0, $et, $line)
		$etime=StringSplit($et, ":") ;result is array
		if (UBound($etime) <> 4) then msgbox (0,"Database Text File error:", "invalid file format found in elapsed on line " & $linecount)
		$elapsed=(int($etime[1])*3600) + (int($etime[2])*60) + int($etime[3])
;		msgbox(0,"elapsed", $elapsed)

		;skip space(s)
		do
		    $charptr+=1
		until ($charptr > stringlen($line) OR stringmid($line, $charptr, 1)<>' ')

		;skip 'elapsed'
		$beginptr=$charptr
		do
		    $charptr+=1
		until ($charptr > stringlen($line) OR stringmid($line, $charptr, 1)==' ')
		if (stringmid($line, $beginptr, $charptr-$beginptr)<>"elapsed") then msgbox(0,"Database Text File error:", "invalid file format found in elapsed tag string on line " & $linecount)

		;skip space(s)
		do
		    $charptr+=1
		until ($charptr > stringlen($line) OR stringmid($line, $charptr, 1)<>' ')
		
		;if we hit an in-file report, skip the line.
		if ($charptr > stringlen($line)) then continueloop

		;read startdate
		$beginptr=$charptr
		Do ;skip until space(s)
			$charptr+=1
		until ($charptr > stringlen($line) OR stringmid($line, $charptr, 1)==' ')
		$startdate=StringMid($line, $beginptr, $charptr-$beginptr) ;result is array
		$etime=StringSplit($startdate, "/")
		if (UBound($etime) <> 4) then msgbox (0,"Database Text File error:", "invalid file format found in startdate on line " & $linecount)

		;skip space(s)
		do
		    $charptr+=1
		until ($charptr > stringlen($line) OR stringmid($line, $charptr, 1)<>' ')

		;read starttime
		$beginptr=$charptr
		Do ;skip until space(s)
			$charptr+=1
		until ($charptr > stringlen($line) OR stringmid($line, $charptr, 1)==' ')
		$starttime=StringMid($line, $beginptr, $charptr-$beginptr) ;result is array
		$etime=StringSplit($starttime, ":")
		if (UBound($etime) <> 4) then msgbox (0,"Database Text File error:", "invalid file format found in starttime on line " & $linecount)
		
		;ignore the rest of the line
		
		;write db
		$err=_SQLite_Exec ( $hDB, "INSERT INTO timeentries(taskname,elapsed,startdate,starttime) VALUES("&_SQLite_Escape($taskname)&","&$elapsed&","&_SQLite_Escape($startdate)&","&_SQLite_Escape($starttime)&");")
		select
		case $err==$SQLITE_OK
				;msgbox(0,"OK","OK")
		case $err==-1
			msgbox(0, "SQLite error", "insert1 prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
		case $err=1
			msgbox(0, "SQLite error", "insert1 problem:Error calling SQLite API 'sqlite3_prepare'")
		case $err=2
			msgbox(0, "SQLite error", "insert1 problem: call prevented by safe mode.")
		endselect
		
		$linecount+=1
	Wend
	FileClose($file)
	setcombo()
EndFunc



Func writedb($filename)
	if ($filename="") Then
		Return
	EndIf
	$file = FileOpen($filename, 2) ;write (overwrite)
	; Check if file opened for reading OK
	If $file = -1 Then
		MsgBox(0, "Database File Error:", "Unable to open file for writing. Some other program may have it open.")
		return
	EndIf
	$linecount=1
	$err=_SQLite_Query ( $hDB, "SELECT taskname,elapsed,startdate,starttime FROM timeentries;", $hQuery )
;	$err=_SQLite_Query ( $hDB, "SELECT taskname,elapsed,startdate,starttime FROM timeentries ORDER BY taskname, startdate, starttime;", $hQuery )
	select
	case $err==$SQLITE_OK
	case $err==-1
		msgbox(0, "SQLite error", "SELECT3 prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
	case $err=1
		msgbox(0, "SQLite error", "SELECT3 problem:Error calling SQLite API 'sqlite3_prepare'")
	case $err=2
		msgbox(0, "SQLite error", "SELECT3 problem: call prevented by safe mode.")
	endselect
	While _SQLite_FetchData ($hQuery, $aRow) = $SQLITE_OK
		$line = '"' & $aRow[0] & '" ' & eltime($aRow[1]) & ' elapsed ' & $aRow[2] & ' ' & $aRow[3] & ' starttime'
		FileWriteLine($file, $line)
		If @error = -1 Then
			msgbox(0, "Database File Error:", "Write Error. on line " & $linecount & "   disk may be full, or file may be readonly, or open by another program.")
			ExitLoop
		endif
		$linecount+=1
	WEnd
	_SQLite_QueryFinalize ($hQuery)

	;--------------
	FileWriteLine($file, "")
	$linecount+=1
	$hq=0
	$err=_SQLite_Query ( $hDB, "SELECT DISTINCT taskname FROM timeentries;", $hQ )
	select
	case $err==$SQLITE_OK
	case $err==-1
		msgbox(0, "SQLite error", "SELECT DISTINCT1 prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
	case $err=1
		msgbox(0, "SQLite error", "SELECT DISTINCT1 problem:Error calling SQLite API 'sqlite3_prepare'")
	case $err=2
		msgbox(0, "SQLite error", "SELECT DISTINCT1 problem: call prevented by safe mode.")
	endselect
	_GUICtrlComboBox_ResetContent( $COMBOTask )
	$change=false
	While _SQLite_FetchData ($hQ, $aRow) = $SQLITE_OK
		$line = '"' & $aRow[0] & '" ' & toteltime($aRow[0]) & ' elapsed'
		FileWriteLine($file, $line)
		If @error = -1 Then
			msgbox(0, "Database File Error:", "Write Error. on line " & $linecount & "   disk may be full, or file may be readonly, or open by another program.")
			ExitLoop
		endif
		$linecount+=1
	WEnd
	_SQLite_QueryFinalize ($hQ)


	FileClose($file)
EndFunc





Func deletetask($tsknm)
	$err=_SQLite_Exec ( $hDB, "DELETE FROM timeentries WHERE taskname="&_SQLite_Escape($tsknm)&";")
	select
	case $err==$SQLITE_OK
	case $err==-1
		msgbox(0, "SQLite error", "DELETE prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
	case $err=1
		msgbox(0, "SQLite error", "DELETE problem:Error calling SQLite API 'sqlite3_prepare'")
	case $err=2
		msgbox(0, "SQLite error", "DELETE problem: call prevented by safe mode.")
	endselect
EndFunc


;------------------------------------------------------GUI------------------------------------------------------------

GUICreate("Task Timer v"&$PROGRAM_VERSION, 350, 360)  ; will create a dialog box

GUICtrlCreateGroup("Database Text I/O File", 5, 5, 340, 80)
$BTNOpen = GUICtrlCreateButton("Open", 5+5, 15+5, 50)
$BTNBrowse = GUICtrlCreateButton("Browse", 5+5 + 50 + 10, 15+5, 60)
$BTNRead = GUICtrlCreateButton("Read", 5+5 + 50 +10 +  60 + 10, 15+5, 50)
$BTNWrite = GUICtrlCreateButton("Write", 5+5 + 50 +10 +  60 + 10 + 50 + 10, 15+5, 50)
;GUICtrlSetTip(-1, "Generate a new password")
;GUICtrlCreateLabel("Size", , , 30)
$TXTDBFile = GUICtrlCreateInput($dbfilename, 5+5, 15+5+30, 320)
;GUICtrlSetState ( -1, $GUI_FOCUS)
GUICtrlSetTip(-1, "The number of characters of the password (width). must be a positive integer.")

$x1=5
$y1=80+10
GUICtrlCreateGroup("New Task", $x1, $y1, 340, 50)
$BTNEnterTask=GUICtrlCreateButton("Enter New Task Name", $x1+5, $y1+15, 122)
$TXTNewTask = GUICtrlCreateInput("", $x1+5+125, $y1+15, 200)
GUICtrlSetState ( -1, $GUI_FOCUS)
GUICtrlSetTip(-1, "task name.")


$x2=5
$y2=$y1+50+10
GUICtrlCreateGroup("Task Work Area", $x2, $y2, 340, 5+5+15+30 + 30+15+5+30 +60)
$COMBOTask=GUICtrlCreateCombo(gentskname(), $x2+5, $y2+15, 330)
$BTNDeleteTask=GUICtrlCreateButton("Delete Task", $x2+5, $y2+15+30, 80)
$BTNStart=GUICtrlCreateButton("Start", $x2+5 + 80 + 10, $y2+15+30, 40)
$BTNStop=GUICtrlCreateButton("Stop", $x2+5 + 80 + 10 + 40 + 10, $y2+15+30, 40)
$LBLRG=GUICtrlCreateLabel(" ", $x2+5 + 80 + 10 + 40 + 10 + 40 + 20, $y2+15+30, 20, 20)
GUICtrlSetBkColor($LBLRG, 0xff0000)
GUICtrlCreateLabel("Elapsed:", $x2+5, $y2+15+30 + 30, 50)
$TXTElapsed = GUICtrlCreateInput(eltime(0), $x2+5+ 50 + 10, $y2+15+30 + 30, 150)
GUICtrlSetStyle($TXTElapsed, $ES_READONLY)
GUICtrlCreateLabel("Total Elapsed:", $x2+5, $y2+15+30 + 30 + 30, 70)
$TXTTotElapsed = GUICtrlCreateInput(eltime(0), $x2+5+ 70 + 10, $y2+15+30 + 30 + 30, 150)
GUICtrlSetStyle($TXTTotElapsed, $ES_READONLY)
GUICtrlCreateLabel("Start Date:", $x2+5, $y2+15+30 + 30 + 30 + 30, 60)
$TXTStartdate = GUICtrlCreateInput("", $x2+5+ 50 + 10, $y2+15+30 + 30 + 30 + 30, 150)
GUICtrlSetStyle($TXTStartdate, $ES_READONLY)
GUICtrlCreateLabel("Start Time:", $x2+5, $y2+15+30 + 30 + 30 + 30 + 30, 60)
$TXTStarttime = GUICtrlCreateInput("", $x2+5+ 50 + 10, $y2+15+30 + 30 + 30 + 30 + 30, 150)
GUICtrlSetStyle($TXTStarttime, $ES_READONLY)


;$TXTO1=GUICtrlCreateInput("", $x2+5, $y2+5+5+15+30 + 30+15+5+30 + 60+ 10, 300)
;$TXTO2=GUICtrlCreateInput("", $x2+5, $y2+5+5+15+30 + 30+15+5+30 + 60+ 10+30, 300)
;$TXTO3=GUICtrlCreateInput("", $x2+5, $y2+5+5+15+30 + 30+15+5+30 + 60+ 10+30+30, 300)
;$TXTO4=GUICtrlCreateInput("", $x2+5, $y2+5+5+15+30 + 30+15+5+30 + 60+ 10+30+30+30, 300)

;readdb($dbfilename)
setcombo()
GUISetState()       ; will display an empty dialog box

; Run the GUI until the dialog is closed
While 1
	$MSG = GUIGetMsg()
	Select
    Case $MSG = $GUI_EVENT_CLOSE; Or $MSG = $BTNEXIT
		;writedb($dbfilename)
		_SQLite_Close($hDB)
		_SQLite_Shutdown ( )
		Exit

	Case $MSG = $BTNOpen
		Run("notepad.exe " & $dbfilename)

	Case $MSG = $BTNRead
		$t=GUICtrlRead($TXTDBFile)
		if (StringStripWS($t, 1 + 2)<>"") then
			$dbfilename=StringStripWS($t, 1 + 2)
			readdb($dbfilename)
		endif
		_GUICtrlComboBox_ShowDropDown ( $COMBOTask, true )


	Case $MSG = $BTNWrite
		$t=GUICtrlRead($TXTDBFile)
		if (StringStripWS($t, 1 + 2)<>"") then
			$dbfilename=StringStripWS($t, 1 + 2)
			writedb($dbfilename)
		endif

	 Case $MSG = $BTNDeleteTask
		$t=getcombo()
		if ($t <> "") then
			if ($timer_running) then
				$timer_running=False
				$elapsed=int(TimerDiff($ts)/1000)
				GUICtrlSetBkColor($LBLRG, 0xff0000) ;red
				;don't bother to write record. it will be deleted
			endif

			deletetask($t)
			;pick new Default for combo
			$taskname=""
			setcombo()
			_GUICtrlComboBox_ShowDropDown ( $COMBOTask, true )
		endif

	Case $MSG = $BTNEnterTask
		;stop timer if it's running & log the event
		if ($timer_running) then
			$timer_running=False
			$elapsed=int(TimerDiff($ts)/1000)
			GUICtrlSetBkColor($LBLRG, 0xff0000) ;red
			;write record
			$err=_SQLite_Exec ( $hDB, "INSERT INTO timeentries(taskname,elapsed,startdate,starttime) VALUES("&_SQLite_Escape($taskname)&","&$elapsed&","&_SQLite_Escape($startdate)&","&_SQLite_Escape($starttime)&");")
			select
			case $err==$SQLITE_OK
			case $err==-1
				msgbox(0, "SQLite error", "insert2 prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
			case $err=1
				msgbox(0, "SQLite error", "insert2 problem:Error calling SQLite API 'sqlite3_prepare'")
			case $err=2
				msgbox(0, "SQLite error", "insert2 problem: call prevented by safe mode.")
			endselect
			;writedb($dbfilename)
		endif
		
		$tsk=StringStripWS(GUICtrlRead($TXTNewTask),1+2)
		$tsk=StringReplace($tsk, '|', "") ;strip pipe characters out.  it mangles the combobox.
		$tsk=StringReplace($tsk, '"', "") ;strip double quotes out.  it ruins the Text File.
		GUICtrlSetData($TXTNewTask, $tsk)
		if ($tsk <> "") then
			;set record attributes that we can
			;but first strip out all quotes.  sqlite doesn't like them.
			$taskname=$tsk
			$elapsed=0
;			;add to combo box
			setcombo()
			;set combo box that this is current selection
			setcombodefault($taskname)
			GUICtrlSetData($TXTElapsed, eltime(0))
			GUICtrlSetData($TXTTotElapsed, eltime(0))
			GUICtrlSetData($TXTStartdate, "")
			GUICtrlSetData($TXTStarttime, "")
		endif
	 Case $MSG = $BTNStart
		 if (not $timer_running AND getcombo()<>"") then
			 ;init the Timer
			 $ts = TimerInit()
			;write start time into record
			$startdate = @MON & '/' & @MDAY & '/' & @YEAR
			$starttime = @HOUR & ':' & @MIN & ':' & @SEC
			GUICtrlSetData($TXTStartdate, $startdate)
			GUICtrlSetData($TXTStarttime, $starttime)
			 $timer_running=true
			
			GUICtrlSetBkColor($LBLRG, 0x00ff00) ;green
		Else
			;timer already running. ignore
			;beep(1000,10)
		endif
	Case $MSG = $BTNStop
		if ($timer_running) then
			;get the elapsed time
			GUICtrlSetBkColor($LBLRG, 0xff0000) ;red
			;write record
			$elapsed=int(TimerDiff($ts)/1000)
			$err=_SQLite_Exec ( $hDB, "INSERT INTO timeentries(taskname,elapsed,startdate,starttime) VALUES("&_SQLite_Escape($taskname)&","&$elapsed&","&_SQLite_Escape($startdate)&","&_SQLite_Escape($starttime)&");")
			select
			case $err==$SQLITE_OK
			case $err==-1
				msgbox(0, "SQLite error", "insert3 prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
			case $err=1
				msgbox(0, "SQLite error", "insert3 problem:Error calling SQLite API 'sqlite3_prepare'")
			case $err=2
				msgbox(0, "SQLite error", "insert3 problem: call prevented by safe mode.")
			endselect
			$id=_SQLite_LastInsertRowID ( $hDB )
			GUICtrlSetData($TXTTotElapsed, toteltime($taskname))
			GUICtrlSetData($TXTElapsed, eltime($elapsed))
			$timer_running=false
			 ;make new record
			 $taskname=getcombo()
		Else
			;timer running - ignore
			;beep(1000,10)
		endif
    Case $MSG = $BTNBrowse
        $var=FileOpenDialog("Database File:", ".", "Text files (*.txt)", 1+8, $dbfilename)
		If @error Then
			;MsgBox(4096,"","No File(s) chosen")
		Else
			$var2 = StringSplit($var, "|")
			$var = $var2[1]
			;MsgBox(4096,"","You chose " & $var)
			if (StringStripWS($var, 1 + 2)<>"") then
				$dbfilename=StringStripWS($var, 1 + 2)
			endif
		EndIf
		GUICtrlSetData($TXTDBFile, $dbfilename)
		
		
	EndSelect
	if ($timer_running) then
		GUICtrlSetData($TXTElapsed, eltime(int(TimerDiff($ts)/1000)))
		GUICtrlSetData($TXTTotElapsed, toteltime($taskname))
	endif
	;GUICtrlSetData($TXTO1, $taskname)
	;GUICtrlSetData($TXTO2, $elapsed)
	;GUICtrlSetData($TXTO3, $startdate)
	;GUICtrlSetData($TXTO4, $starttime)
	;-------detect if user changed the combo box.
	;this is not a GUI event unfortunately.
	$tsk=getcombo()
	if ($tsk <> "" AND $tsk<>$taskname) then
		;stop timer if it's running & log the event
		if ($timer_running) then
			$timer_running=False
			$elapsed=int(TimerDiff($ts)/1000)
			GUICtrlSetBkColor($LBLRG, 0xff0000) ;red
			;write db
			$err=_SQLite_Exec ( $hDB, "INSERT INTO timeentries(taskname,elapsed,startdate,starttime) VALUES("&_SQLite_Escape($taskname)&","&$elapsed&","&_SQLite_Escape($startdate)&","&_SQLite_Escape($starttime)&");")
			select
			case $err==$SQLITE_OK
			case $err==-1
				msgbox(0, "SQLite error", "insert4 prob: Error Code: " & _SQLite_ErrCode() & @CR & "Error Message: " & _SQLite_ErrMsg())
			case $err=1
				msgbox(0, "SQLite error", "insert4 problem:Error calling SQLite API 'sqlite3_prepare'")
			case $err=2
				msgbox(0, "SQLite error", "insert4 problem: call prevented by safe mode.")
			endselect

			;writedb($dbfilename)
		endif
;		;make new record
;		$index=$count
;		$count+=1
;		$maxarray+=1
;		;redim arrays
;		redim $taskname[$maxarray], $elapsed[$maxarray], $startdate[$maxarray], $starttime[$maxarray]
;		;add to database in memory
		$taskname=$tsk
		GUICtrlSetData($TXTElapsed, eltime(0))
		GUICtrlSetData($TXTTotElapsed, toteltime($tsk)) ;$taskname
		GUICtrlSetData($TXTStartdate, "")
		GUICtrlSetData($TXTStarttime, "")
    endif
	if ($tsk <> "" AND $tsk==$taskname) then
		GUICtrlSetData($TXTTotElapsed, toteltime($tsk)) ;$taskname
	endif
	;sleep(500)
Wend
